﻿using System;
using System.Linq;
using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using MVR.FileManagementSecure;

namespace SPQR {
	public class SoundRefresh : MVRScript {
		
		bool sceneLoaded = false;
		DateTime lastTime;
		JSONStorableString filter;
		public override void Init() {
			if(containingAtom.type != "Person")
            {
				SuperController.LogError("SoundRefresh plugin must be added to a Person atom");
				return;
            }
			InitUI();
			InvokeRepeating("UpdateSecond", 1f, 1f);
			InvokeRepeating("UpdateConstant", 1f, 1f / 30f);
			InvokeRepeating("UpdateConstant5", 1f, 1f / 5f);
			SuperController.singleton.onSceneLoadedHandlers += OnSceneLoaded;
		}
		void InitUI()
        {
            filter = new JSONStorableString("Filter", "Custom\\Assets\\Audio\\RT_Lipsync\\latest.wav", UpdatedFilter);
            var t = CreateTextInput_p67(this, null, filter, false)[0];
            RegisterString(filter);

			var btn1 = CreateButton("Play", false);
			btn1.button.onClick.AddListener(RefreshSounds);

			var act1 = new JSONStorableAction("Play", RefreshSounds);
			RegisterAction(act1);
		}

		void Start() {
			UpdateDateTime();
		}

	
		void Update() {
		
		}

	
		void FixedUpdate() {
		
		}
		void UpdateSecond()
		{
			CheckFileChange();
			if (IsDisabled()) return;

		}
		void UpdateConstant()
		{
			if (IsDisabled()) return;

		}
		void UpdateConstant5()
		{
			if (IsDisabled()) return;

		}
		void OnDestroy()
		{
			CancelInvoke();
			SuperController.singleton.onSceneLoadedHandlers -= OnSceneLoaded;
		}

		void OnSceneLoaded()
		{
			sceneLoaded = true;
		}
		bool IsDisabled()
        {
			if (!enabledJSON.val)
			{
				return true;
			}
			return false;
		}
		void UpdatedFilter(string s)
        {
			UpdateDateTime();

		}
		bool FileIsOk()
        {
			if (filter.val == "") return false;
			return FileManagerSecure.FileExists(filter.val);

		}

		void CheckFileChange()
        {
			if (!FileIsOk()) return;
			var currentTime = FileManagerSecure.FileLastWriteTime(filter.val);
			//SuperController.LogMessage(lastTime.ToString() + " vs " + currentTime.ToString() +" = " + DateTime.Compare(lastTime, currentTime));
			if(DateTime.Compare(lastTime, currentTime) < 0){
				SuperController.LogMessage(" CHANGE at " + currentTime.ToString());
				RefreshSounds();
				UpdateDateTime();
            }
		}
		void UpdateDateTime()
        {
			if (!FileIsOk()) return;
			lastTime = FileManagerSecure.FileLastWriteTime(filter.val);
        }
		public static UIDynamicTextField[] CreateTextInput_p67(MVRScript X, JSONStorableString labeljss, JSONStorableString s, bool rightSide = false)
		{
			var response = new UIDynamicTextField[2];
			UIDynamicTextField dtext;
			if (labeljss != null)
			{
				dtext = X.CreateTextField(labeljss, rightSide);
				dtext.height = 5f;
				dtext.backgroundColor = Color.clear;
				response[1] = dtext;
			}

			var textfield = X.CreateTextField(s, rightSide);
			textfield.backgroundColor = Color.white;
			var input = textfield.gameObject.AddComponent<InputField>();
			input.textComponent = textfield.UItext;
			s.inputField = input;


			response[0] = textfield;

			return response;
		}

		void RefreshSounds()
        {
			string localPath = SuperController.singleton.NormalizeLoadPath(filter.val);
			if (!FileManagerSecure.FileExists(localPath)) return;

			var namesplit = localPath.Split('/');
			var name = "\\"+namesplit[namesplit.Length - 1];

			var j = URLAudioClipManager.singleton.GetJSON();
			foreach (var k in j["clips"].AsArray)
			{
				var clip = (JSONNode)k;

				if(clip["url"].Value  == localPath)
                {
					var nac = URLAudioClipManager.singleton.GetClip(clip["displayName"]);
					URLAudioClipManager.singleton.RemoveClip(nac);
				}
			}
			var raeAudioSource = containingAtom.GetStorableByID("HeadAudioSource") as AudioSourceControl;
			var qclip = URLAudioClipManager.singleton.QueueClip(localPath, name);
			var nacclip = URLAudioClipManager.singleton.GetClip(localPath);
			if(nacclip.sourceClip == null)
            {
				RaeSoundDelayed_p67(localPath,raeAudioSource,true,0);
			} else
            {
				raeAudioSource.audioSource.clip = nacclip.sourceClip;
				raeAudioSource.audioSource.Play();
			}
		

		}

		public void RaeSoundDelayed_p67(string localPath, AudioSourceControl raeAudioSource, bool force, int ite)
		{
			if (ite > 300)
			{
				SuperController.LogMessage("SoundRefresh expired - couldn't load "+filter.val);
				return;
			}

			var nac = URLAudioClipManager.singleton.GetClip(localPath);
			if (nac.sourceClip == null)
			{
				StartCoroutine(InvokeRoutine_p67(() => RaeSoundDelayed_p67(localPath, raeAudioSource, force, ite + 1), 0.05f));
				return;
			}
			raeAudioSource.audioSource.clip = nac.sourceClip;
			if (force)
			{
				raeAudioSource.StopAndClearQueue();
			}
			raeAudioSource.audioSource.Play();
			
		}

		IEnumerator InvokeRoutine_p67(System.Action f, float delay)
		{
			yield return new WaitForSeconds(delay);
			f();
		}

	}
}